// Copyright 2013 Krzysztof Czainski
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

/*
 * @file  copy.hpp
 *
 * @since  2013-11-29
 * @author  Krzysztof Czainski <1czajnik@gmail.com>
 */

#ifndef COPY_HPP_
#define COPY_HPP_

namespace boost {

/// @ingroup boost_move
//@{

/// This function returns a copy of @p x obtained by explicitly calling the copy constructor,
/// and it is the default implementation of an ADL customization point called from boost::copy().

template < class T >
T copy_adl( T const& x )
{
    return T(x); // explicitly call the copy constructor
}

/// Makes an explicit copy of @p x.
///
/// This function simply makes an unqualified call: @code return copy_adl(x); @endcode so you can
/// provide an overload of `copy_adl()` with your type to customize behavior of `boost::copy()`.
/// See `boost::copy_adl()` for the default behavior.
///
/// For example, copy_adl() is overloaded for cloneable_ptr_t in boost::smart_ptr::copy_adl().
template < class T >
T copy( T const& x )
{ return copy_adl(x); }

//@}

} // boost

#endif // COPY_HPP_
