// Copyright 2013 Krzysztof Czainski
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

/*
 * @file  CopyMoveDemo.hpp
 *
 * @since  2013-11-28
 * @author  Krzysztof Czainski <1czajnik@gmail.com>
 */

#ifndef COPYMOVEDEMO_HPP_
#define COPYMOVEDEMO_HPP_

#include <boost/move/core2.hpp>

#include <boost/move/utility.hpp>

#include "using_iostream.hpp"

namespace def {

//[CopyMoveDemo]
struct CopyMoveDemo
{
    BOOST_MOVABLE2(CopyMoveDemo)
public:
    CopyMoveDemo() { cout << "CopyMoveDemo()" << endl; }
    explicit CopyMoveDemo( int ) { cout << "CopyMoveDemo(int)" << endl; }
    CopyMoveDemo( char const* ) { cout << "CopyMoveDemo(str)" << endl; }
    CopyMoveDemo( CopyMoveDemo const& ) { cout << "CopyMoveDemo(const&)" << endl; }
    CopyMoveDemo( BOOST_RV_REF(CopyMoveDemo) ) { cout << "CopyMoveDemo(&&)" << endl; }
    ~CopyMoveDemo() { cout << "~CopyMoveDemo()" << endl; }
    friend void swap( CopyMoveDemo& a, CopyMoveDemo& b ) BOOST_NOEXCEPT
    { cout << "swap CopyMoveDemo" << endl; }
};
//[CopyMoveDemo]

} // def

using def::CopyMoveDemo;

//[CopyMoveDemo_alt]
namespace alt {
struct CopyMoveDemo
{
    BOOST_COPYABLE_AND_MOVABLE(CopyMoveDemo)
public:
    CopyMoveDemo() { cout << "CopyMoveDemo()" << endl; }
    explicit CopyMoveDemo( int ) { cout << "CopyMoveDemo(int)" << endl; }
    CopyMoveDemo( char const* ) { cout << "CopyMoveDemo(str)" << endl; }
    CopyMoveDemo( CopyMoveDemo const& ) { cout << "CopyMoveDemo(const&)" << endl; }
    CopyMoveDemo( BOOST_RV_REF(CopyMoveDemo) ) { cout << "CopyMoveDemo(&&)" << endl; }
    ~CopyMoveDemo() { cout << "~CopyMoveDemo()" << endl; }
    CopyMoveDemo& operator=( BOOST_COPY_ASSIGN_REF(CopyMoveDemo) )
    { cout << "CopyMoveDemo=const&\n  clean this\n  copy arg" << endl; return *this; }
    CopyMoveDemo& operator=( BOOST_RV_REF(CopyMoveDemo) )
    { cout << "CopyMoveDemo=&&\n  clean this\n  move arg" << endl; return *this; }
#if 0
    CopyMoveDemo& operator=( char const* )
    { cout << "CopyMoveDemo=str\n  clean this\n  copy arg" << endl; return *this; }
#endif // 0
};
} // alt
//[CopyMoveDemo_alt]

#endif // COPYMOVEDEMO_HPP_
