// Copyright 2013 Krzysztof Czainski
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

/*
 * @file  Matrix.hpp
 *
 * @since  2013-11-28
 * @author  Krzysztof Czainski <1czajnik@gmail.com>
 */

#include "CopyMoveDemo.hpp"
#include "using_iostream.hpp"

//[Matrix]
class Matrix
{
    // std::vector<double> data_;
    CopyMoveDemo data_;

    BOOST_MOVABLE2(Matrix)
    BOOST_DELETED_COPY_CONSTRUCTOR(Matrix)

public:

    Matrix() {} // = default;
    Matrix( BOOST_RV_REF(Matrix) b ) : data_( boost::move(b.data_) ) {} // = default;
    friend void swap( Matrix& a, Matrix& b ) { swap( a.data_, b.data_ ); }

    Matrix& operator+=( Matrix const& b ) { cout << "+=" << endl; return *this; }
};

inline Matrix operator+( Matrix a, Matrix const& b ) { a += b; return boost::move(a); }
//[Matrix]

//[main]
int main()
{
    Matrix a, b, c;
    cout << "#1#" << endl;
    Matrix d = boost::move(b) + c;
    cout << "#2#" << endl;
    Matrix e = boost::move(b) + c + d;
    cout << "#3#" << endl;
    Matrix f = ( boost::move(c) + d ) + b;
    cout << "###" << endl;
}
//[main]
